#!/usr/bin/env python

"""
Moin wiki serialisers.

Copyright (C) 2017 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from cgi import escape

class Serialiser:

    "General serialisation support."

    def __init__(self, out):
        self.out = out

class MoinSerialiser(Serialiser):

    "Serialisation of the page."

    def start_region(self, level, indent, type):
        out = self.out
        if level:
            out(" " * indent + "{" * level)
        if type and level:
            out("#!%s\n" % type)

    def end_region(self, level, indent, type):
        out = self.out
        if level:
            out("}" * level)

    def start_block(self):
        pass

    def end_block(self):
        pass

    def start_defitem(self, pad, extra):
        self.out((extra and "\n" + extra + "::" or "") + pad)

    def end_defitem(self, pad, extra):
        pass

    def start_defterm(self, pad):
        self.out(pad)

    def end_defterm(self, pad):
        self.out("::")

    def start_emphasis(self):
        self.out("''")

    def end_emphasis(self):
        self.out("''")

    def start_heading(self, level, extra, pad):
        self.out(extra + "=" * level + pad)

    def end_heading(self, level, pad, extra):
        self.out(pad + "=" * level + extra)

    def start_larger(self):
        self.out("~+")

    def end_larger(self):
        self.out("+~")

    def start_listitem(self, indent, marker, space):
        self.out("%s%s%s" % (indent * " ", marker, space))

    def end_listitem(self, indent, marker):
        pass

    def start_monospace(self):
        self.out("`")

    def end_monospace(self):
        self.out("`")

    def start_smaller(self):
        self.out("~-")

    def end_smaller(self):
        self.out("-~")

    def start_strong(self):
        self.out("'''")

    def end_strong(self):
        self.out("'''")

    def start_subscript(self):
        self.out(",,")

    def end_subscript(self):
        self.out(",,")

    def start_superscript(self):
        self.out("^")

    def end_superscript(self):
        self.out("^")

    def start_table_cell(self):
        pass

    def end_table_cell(self):
        self.out("||")

    def start_table_row(self):
        self.out("||")

    def end_table_row(self, trailing):
        self.out(trailing)

    def start_underline(self):
        self.out("__")

    def end_underline(self):
        self.out("__")

    def break_(self):
        self.out("\n")

    def rule(self, length):
        self.out("-" * length)

    def text(self, s):
        self.out(s)

class HTMLSerialiser(Serialiser):

    "Serialisation of the page."

    def start_region(self, level, indent, type):
        l = []
        out = l.append
        if level:
            out("level-%d" % level)

        if indent:
            out("indent-%d" % indent)

        # NOTE: Encode type details for CSS.

        if type:
            out("type-%s" % escape(type, True))

        self.out("<span class='%s'>" % " ".join(l))

    def end_region(self, level, indent, type):
        self.out("</span>")

    def start_block(self):
        self.out("<p>")

    def end_block(self):
        self.out("</p>")

    def start_defitem(self, pad, extra):
        self.out("<dd>")

    def end_defitem(self, pad, extra):
        self.out("</dd>")

    def start_defterm(self, pad):
        self.out("<dt>")

    def end_defterm(self, pad):
        self.out("</dt>")

    def start_emphasis(self):
        self.out("<em>")

    def end_emphasis(self):
        self.out("</em>")

    def start_heading(self, level, extra, pad):
        self.out("<h%d>" % level)

    def end_heading(self, level, pad, extra):
        self.out("</h%d>" % level)

    def start_larger(self):
        self.out("<big>")

    def end_larger(self):
        self.out("</big>")

    def start_listitem(self, indent, marker, space):
        self.out("<li>")

    def end_listitem(self, indent, marker):
        self.out("</li>")

    def start_monospace(self):
        self.out("<tt>")

    def end_monospace(self):
        self.out("</tt>")

    def start_smaller(self):
        self.out("<small>")

    def end_smaller(self):
        self.out("</small>")

    def start_strong(self):
        self.out("<strong>")

    def end_strong(self):
        self.out("</strong>")

    def start_subscript(self):
        self.out("<sub>")

    def end_subscript(self):
        self.out("</sub>")

    def start_superscript(self):
        self.out("<sup>")

    def end_superscript(self):
        self.out("</sup>")

    def start_table_cell(self):
        self.out("<td>")

    def end_table_cell(self):
        self.out("</td>")

    def start_table_row(self):
        self.out("<tr>")

    def end_table_row(self, trailing):
        self.out("</tr>")

    def start_underline(self):
        self.out("<span style='text-decoration: underline'>")

    def end_underline(self):
        self.out("</span>")

    def break_(self):
        pass

    def rule(self, length):
        self.out("<hr style='height: %dpt' />" % min(length, 10))

    def text(self, s):
        self.out(escape(s))

# Top-level functions.

def serialise(doc, serialiser=MoinSerialiser):
    l = []
    doc.to_string(serialiser(l.append))
    return "".join(l)

# vim: tabstop=4 expandtab shiftwidth=4
